// This version Pipmaker_V13_1 by Enforcer has been modified from the PipMakerV10.
// All orders are now closed when ProfitTarget is reached to eliminate outlying orders from building up negative equity.
// Multiplier/CounterTrendMultiplier now acts as Martingale - doubling lot size at Spacing pips away in opposite trend from initial order (CounterTrendMultiplier) or
// at TrendSpacing intervals in same direction of initial order if set to "1".
// Added option for a stop loss on each order but not profitable when used in backtests.
// This EA allows for both Buy and Sell orders at the same time in the same currency which will act as a hedge until the prevailing trend resumes thus limiting
// drawdown.  This obviously won't work if your broker does not allow hedging.
//Enforcer's addons/changes/fixes:
//- Loss Management - orders that fall below max alowed loss are closed and optionally a recovery order is placed
//- Added iTrend indicator
//- Added reverse signal option
// - Added more take profit methods - close all orders when profit reach profit target (old mode), close if buys or sells reach 
// profit target and mixed mode, which one comes first (best)
// - Added MoneyManagement, wii calculate lot size, lot increment and profit target
// - Replaced DPO with TMA (triple moving average)
// - Reworked CCI to trigger at specified level
// - Added Forced start option
// - Added AutoSpacing, default spacing will be calculated by StdDev
// - removed VarySpacing by number of orders
// - Added ARSI indicator (Adaptive RSI - external)
// - Added ProfitTrailing option
// - added ProfitSkew option - will modify default ProfitTarget when MoneyManagement=true
// - added ATR as option for ARSI trigger

#include <stdlib.mqh>
#include <stderror.mqh>
#define  NL    "\n"

// Regular variables
extern bool    TradeShort              = true; //Allow place sell ordes
extern bool    TradeLong               = true; //Allow place buy orders
extern int     OpenOnTick              = 0;

extern bool    UseTMA                  = false; //Triple moving average
extern int     MAperiod                = 60;

extern bool    UseARSI                = true;  //Adaptive RSI
extern int     RSI_period             = 5;    // TF for ARSI
extern int     RSI_bars               = 28;    // lenght of ARSI
extern double  ARSI_trigger           = 0; // level to trigger trade. If = 0 will be used ATR as automatic trigger level

extern bool    UseCCI                 = false;
extern int     CCI_Period             = 60; // CCi time frame
extern int     CCI_lenght             = 60; //CCI bars
extern int     cci_trigger            = 25; // +/- level. lower value = enter in trade faster

extern bool    Use_iTrend              = true;
extern int     iTrendPeriod            = 5; //iTrend TF

extern bool    ReverseDirection        = false; // true = will trade long when price is low and short and price is high. false = will trade long when price is high and short when price is low


extern bool    MoneyMangement          = true; //will autocalculate lot size, lot increment and profit target
extern bool    MicroAccount            = true; //will divide LotSize by 10
extern double  MaximumRisk             = 0.5;  // 1 mean 0.1% of balance
extern double  LotSize                 = 0.02;
extern double  LotIncrement            = 0.01;
extern double  Multiplier              = 0;  // Will increase orders in Martingale fashion in direction of trend if set to "1". Used with TrendSpacing only.
extern double  CounterTrendMultiplier  = 0;  // Will increase orders in Martingale fashion in opposite direction of trend if set to "1". Used with Spacing only.

extern double  ProfitTarget            = 4;  // All orders closed when this profit target amount (in dollars) is reached
extern double  ProfitSkew              = 2;  // Increase/decrease ProfitTarget when MoneyMangement=true
extern int     ProfitMode              = 0;  // 1= mixed mode, 2= global mode, 3= split mode
extern bool    ProfitTrailing          = true;  // Will try to allow profit grow beyond ProfitTarget
extern double  MaxRetrace              = 25;  // auto calculated Maximum percent of MaxProfit allowed to decrease before close all orders

extern int     ForcedStart             = 0;  // 1 = New cycle will start if signal is for long, 2 = New cycle will start if signal is for short, 0 = disabled
extern double  SL                      = 0;  // Performs better with no initial stoploss.

extern bool    AutoSpacing             = true;  //Spacing will be calculated using stdDev
extern int     StDevTF                 = 60;    // TF for StDev
extern int     StDevPer                = 3;    // lenght of StDev
extern int     StDevMode               = MODE_EMA; // mode of StDev - 0=SMA, 1=EMA, 2=SMMA, 3=LWMA 
extern int     StDevTF2                 = 15;    // TF for StDev
extern int     StDevPer2                = 3;    // lenght of StDev
extern int     StDevMode2               = MODE_EMA; // mode of StDev - 0=SMA, 1=EMA, 2=SMMA, 3=LWMA

extern int     Spacing                 = 15; // Minimum distance of orders placed against the trend of the initial order, In effect only if AutoSpacing=false
extern int     TrendSpacing            = 15; // Minimum distance of orders placed with the trend of the initial order (set to 1000 to disable )

extern int     CloseDelay              = 190; // Minimum close time for ActivTrades to not be considered scalping
extern bool    CeaseTrading            = false;
extern string  PauseTrading            ="Pause Trading at Timeinterval";
extern int     StartTime               = 0; //Example: Trading pause starts at day 15 hour 14, minute 30 (server time)--> input= 151430
extern int     EndTime                 = 0; //Example: Trading pause ends at day 15 hour 15, minute 10 (server time)--> input= 151510
extern string  QuitTrading             = "Quit Trading at Time";
extern int     endDayHourMinute        = 0; //Example: Quit trading on day 17 hour 21 minute 59 (server time)-->input=172159
extern bool    RightSideLabel          = false;
extern int     SessionTarget           = 100000; //Trading will be stopped if this amount has been earned in this session
extern int     MaximumBuyOrders        = 20;
extern int     MaximumSellOrders       = 20;
 
extern string  LossManagement         = "What to do if things are going wrong";
extern string  Warning                = "This feature is in early stage and is not finished!";
extern bool    AllowRecovery          = true;   //enable/disable loss management
extern double  RecActivation          = 250;      //maximum loss in balance procents 1=1% max loss
extern double  MaxLossPercent2        = 10;      //maximum loss in balance procents 1=1% max loss
extern bool    ExitAllTrades          = false;   //Close all open orders
extern bool    StopTrading            = false;   //stop trading if loss management was triggered
extern bool    PlaceRecoveryOrders    = true;   //Use counter orders to get in profit
extern int     MaxRecoveryOrders      = 2;      //max extra orders to use for recovery
extern double  RecoveryTakeProfit     = 20;      //take profit in points
extern double  RecoveryStopLoss       = 999;     //stop loss in points
extern double  RecoveryLotMultiplier  = 1;     //lot size is auto calculated for USD based pairs. Increase or decrease according to pair used.
extern double  RecoveryLotMultiplier2 = 1;     //lot size is auto calculated for USD based pairs. Increase or decrease according to pair used.
extern bool    RecReverse             = false;  // reverse direction of recovery trading
extern bool    SavingAccount          = false;   
extern int     MaxLossAmount          = 750;
extern bool    increasebyloss         = false;
extern double  increaselimit          = 500;
extern double  increasefactor         = 1.5;
extern bool    AllowHedging           = true;
extern bool    ChangeLotsAfterRec     = true;
// Internal settings
//int            Step           = 1;
string EA_name                = "PipMaker Hedging The Hedge";
double         stoploss       = 0;
int            Error          = 0;
int            Order          = 0;
int            Slippage       = 3;
int            Reference      = 0;
string         TradeComment;
double         TickPrice      = 0;

int            MaxBuys        = 0;
int            MaxSells       = 0;
double         MaxProfit      = 0;
bool           Auditing       = false;
string         Filename;
double         initialBalance;
int            lotPrecision;
int            StopLoss=100;
bool           TradeAllowed   = true;

int init()
{
   if (Symbol() == "AUDCADm" || Symbol() == "AUDCAD") Reference = 801001;
   if (Symbol() == "AUDJPYm" || Symbol() == "AUDJPY") Reference = 801002;
   if (Symbol() == "AUDNZDm" || Symbol() == "AUDNZD") Reference = 801003;
   if (Symbol() == "AUDUSDm" || Symbol() == "AUDUSD") Reference = 801004;
   if (Symbol() == "AUDCHFm" || Symbol() == "AUDCHF") Reference = 801020;  
   if (Symbol() == "CHFJPYm" || Symbol() == "CHFJPY") Reference = 801005;
   if (Symbol() == "EURAUDm" || Symbol() == "EURAUD") Reference = 801006;
   if (Symbol() == "EURCADm" || Symbol() == "EURCAD") Reference = 801007;
   if (Symbol() == "EURCHFm" || Symbol() == "EURCHF") Reference = 801008;
   if (Symbol() == "EURGBPm" || Symbol() == "EURGBP") Reference = 801009;
   if (Symbol() == "EURJPYm" || Symbol() == "EURJPY") Reference = 801010;
   if (Symbol() == "EURUSDm" || Symbol() == "EURUSD") Reference = 801011;
   if (Symbol() == "GBPCHFm" || Symbol() == "GBPCHF") Reference = 801012;
   if (Symbol() == "GBPJPYm" || Symbol() == "GBPJPY") Reference = 801013;
   if (Symbol() == "GBPUSDm" || Symbol() == "GBPUSD") Reference = 801014;
   if (Symbol() == "NZDJPYm" || Symbol() == "NZDJPY") Reference = 801015;
   if (Symbol() == "NZDUSDm" || Symbol() == "NZDUSD") Reference = 801016;
   if (Symbol() == "USDCHFm" || Symbol() == "USDCHF") Reference = 801017;
   if (Symbol() == "USDJPYm" || Symbol() == "USDJPY") Reference = 801018;
   if (Symbol() == "USDCADm" || Symbol() == "USDCAD") Reference = 801019;
   if (Reference == 0) Reference = 801999;
   initialBalance= AccountBalance();   
   TradeComment  = StringConcatenate(Symbol()," ",Period()," ",EA_name);      
   Filename = StringConcatenate(EA_name+"_",Symbol(),"_",Period(),"_M",".txt");
   CalculateLotPrecision();
   return(0);
}

int deinit()
{
   if(ObjectFind("MidPoint")==0){
      ObjectDelete("MidPoint");      
   }
   if(ObjectFind("MarginPercent")==0){
      ObjectDelete("MarginPercent");
   }
   if(ObjectFind("LowMarginPercent")==0){
      ObjectDelete("LowMarginPercent");
   }   
   Comment("");
   return(0);
}

void CalculateLotPrecision(){
   double lotstep=MarketInfo(Symbol(),MODE_LOTSTEP);
   if(lotstep==1) lotPrecision=0;
   if(lotstep==0.1) lotPrecision=1;
   if(lotstep==0.01) lotPrecision=2;
   if(lotstep==0.001) lotPrecision=3;
}

double AutoLot()
  {
   double lot;
   //lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/10000.0,2);
   lot=NormalizeDouble(AccountBalance()/10000/10,lotPrecision)*MaximumRisk;
   if(MicroAccount)lot=NormalizeDouble(lot/10/10,lotPrecision);

   if(lot<0.01) lot=0.01;
   return(lot);
  }


int RecoveryOrders(){
int RecTotal=0;
int cnt;
   for(cnt=0;cnt<OrdersTotal();cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == Reference * 2 && OrderComment()=="Rec" && OrderSymbol()==Symbol())
      RecTotal++;
   }
   return (RecTotal);
}
int LastBuyRec(){
int lasttime=0;
int cnthe;
   for(cnthe=0;cnthe<OrdersTotal();cnthe++)
   {
      OrderSelect(cnthe, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() == OP_BUY && OrderMagicNumber() == Reference * 2 && OrderComment()=="Rec" && OrderSymbol()==Symbol())
      {
      if(OrderOpenTime()>lasttime)lasttime=OrderOpenTime();
      }
   }
   return (lasttime);
}

int LastSellRec(){
int lasttime=0;
int cnthe;
   for(cnthe=0;cnthe<OrdersTotal();cnthe++)
   {
      OrderSelect(cnthe, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() == OP_SELL && OrderMagicNumber() == Reference * 2 && OrderComment()=="Rec" && OrderSymbol()==Symbol())
      {
      if(OrderOpenTime()>lasttime)lasttime=OrderOpenTime();
      }
   }
   return (lasttime);
}

int Lastbuytrade(){
int lasttimen=0;
int cnthen;
   for(cnthen=0;cnthen<OrdersTotal();cnthen++)
   {
      OrderSelect(cnthen, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()==OP_BUY && OrderMagicNumber() == Reference && OrderSymbol()==Symbol())
      {
      if(OrderOpenTime()>lasttimen)lasttimen=OrderOpenTime();
      }
   }
   return (lasttimen);
}
int Lastselltrade(){
int lasttimen=0;
int cnthen;
   for(cnthen=0;cnthen<OrdersTotal();cnthen++)
   {
      OrderSelect(cnthen, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()==OP_SELL && OrderMagicNumber() == Reference && OrderSymbol()==Symbol())
      {
      if(OrderOpenTime()>lasttimen)lasttimen=OrderOpenTime();
      }
   }
   return (lasttimen);
}


int SumBuy(){
int Buy;
for(int ih=0;ih<OrdersTotal();ih++)
{
     OrderSelect(ih,SELECT_BY_POS, MODE_TRADES);
     if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference && OrderType() == OP_BUY)
     Buy+=OrderLots()*OrderOpenPrice()*10000000;
     }
return (Buy);
}

int SumSell(){
int Sell;
for(int ihs=0;ihs<OrdersTotal();ihs++)
{
     OrderSelect(ihs,SELECT_BY_POS, MODE_TRADES);
     if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference && OrderType() == OP_SELL)
     Sell+=OrderLots()*OrderOpenPrice()*10000000;
     }
return (Sell);
}

void PlaceBuyOrder()
{
   double BuyOrders, Lots;
   double LowestBuy = 1000, HighestBuy;
   
   if(MoneyMangement)
   {
    LotSize=AutoLot();
   if(LotIncrement>0) LotIncrement=LotSize;
    }
   
   TickPrice = 0;

   RefreshRates();
   
   for (Order = OrdersTotal() - 1; Order >= 0; Order--)
   {
      if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference && OrderType() == OP_BUY)
         {
            if (OrderOpenPrice() < LowestBuy) LowestBuy = OrderOpenPrice();
            if (OrderOpenPrice() > HighestBuy) HighestBuy = OrderOpenPrice();
            BuyOrders++;
         }
      }
   }

   if (Ask >= HighestBuy + (TrendSpacing * Point))
   {
      if (Multiplier == 1)
         Lots = NormalizeDouble(MathPow(2,BuyOrders)*LotSize, lotPrecision);
      else
         Lots = NormalizeDouble(LotSize + (LotIncrement * BuyOrders), lotPrecision);
   }

   if (Ask <= LowestBuy - (Spacing * Point))
   {
      if (CounterTrendMultiplier == 1)
         Lots = NormalizeDouble(MathPow(2,BuyOrders)*LotSize, lotPrecision);
      else
         Lots = NormalizeDouble(LotSize + (LotIncrement * BuyOrders), lotPrecision);
   }

   if(BuyOrders==0)
   {
      Lots = NormalizeDouble(LotSize, lotPrecision);
   }

   if (Lots == 0)
   {
      if (Multiplier == 1)
         Lots = NormalizeDouble(LotSize, lotPrecision);
      else
         Lots = NormalizeDouble(LotSize, lotPrecision);
   }
   
   
   if( (BuyOrders < MaximumBuyOrders))
   {
      if (SL == 0) stoploss = 0; else stoploss = Ask - (SL * Point);
   }

   Error = GetLastError();
   if (Error != 0)
      Write("Error opening BUY order: " + ErrorDescription(Error) + " (C" + Error + ")  Ask:" + Ask + "  Slippage:" + Slippage);
   else
   {
      TickPrice = Close[0];
   }

}

void PlaceSellOrder()
{
   double SellOrders, Lots;
   double HighestSell, LowestSell = 1000;
   
   if(MoneyMangement)
   {
    LotSize=AutoLot();
   if(LotIncrement>0) LotIncrement=LotSize;
   }
   
   TickPrice = 0;

   RefreshRates();
   
   for (Order = OrdersTotal() - 1; Order >= 0; Order--)
   {
      if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference && OrderType() == OP_SELL)
         {
            if (OrderOpenPrice() > HighestSell) HighestSell = OrderOpenPrice();
            if (OrderOpenPrice() < LowestSell) LowestSell = OrderOpenPrice();
            SellOrders++;
         }
      }
   }

   if (Bid <= LowestSell - (TrendSpacing * Point))
   {
//         if (Multiplier)
      if (Multiplier == 1)
         Lots = NormalizeDouble(MathPow(2,SellOrders)*LotSize, lotPrecision);
      else
         Lots = NormalizeDouble(LotSize + (LotIncrement * SellOrders), lotPrecision);
   }
   
   if (Bid >= HighestSell + (Spacing * Point))
   {
//         if (Multiplier)
      if (CounterTrendMultiplier == 1)
         Lots = NormalizeDouble(MathPow(2,SellOrders)*LotSize, lotPrecision);
      else
        Lots = NormalizeDouble(LotSize + (LotIncrement * SellOrders), lotPrecision);
   }

   if(SellOrders==0)
   {
      Lots = NormalizeDouble(LotSize, lotPrecision);
   }

   if (Lots == 0)
   {
//         if (Multiplier) 
      if (Multiplier == 1)
         Lots = NormalizeDouble(LotSize, lotPrecision);
      else
         Lots = NormalizeDouble(LotSize, lotPrecision);
   }
   
   if((SellOrders < MaximumSellOrders))
   {  
      if (SL == 0) stoploss = 0; else stoploss = Bid + (SL * Point);
      

   }
   
   Error = GetLastError();
   if (Error != 0)
      Write("Error opening SELL order: " + ErrorDescription(Error) + " (D" + Error + ")  Bid:" + Bid + "  Slippage:" + Slippage);
   else
   {
      TickPrice = Close[0];
   }
}

void CloseAllBuyProfit()
{
int spread=MarketInfo(Symbol(),MODE_SPREAD);
   for(int i = OrdersTotal()-1; i >=0; i--)
       {
       OrderSelect(i, SELECT_BY_POS);
       bool result = false;
       if (OrderSymbol()==Symbol() && OrderMagicNumber() == Reference && OrderType() == OP_BUY)  
         {
           if (TimeCurrent()-OrderOpenTime() >= CloseDelay) result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), 0, Aqua );
         }
       }
     
  return; 
}

void CloseAllSellProfit()
{
int spread=MarketInfo(Symbol(),MODE_SPREAD);
   for(int i = OrdersTotal()-1; i >=0; i--)
      {
      OrderSelect(i, SELECT_BY_POS);
      bool result = false;
      if (OrderSymbol()==Symbol() && OrderMagicNumber() == Reference && OrderType() == OP_SELL) 
       {
         if (TimeCurrent()-OrderOpenTime() >= CloseDelay) result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), 0, Lime );
       }
     }
 
  return; 
}

int start()
{
   double         MarginPercent;
   static double  LowMarginPercent = 10000000, LowEquity = 10000000;
   double         BuyPipTarget, SellPipTarget;
   int            SellOrders, BuyOrders,SellOrdersRec, BuyOrdersRec, MaxBuysRec, MaxSellsRec; 
   double         BuyPips, SellPips, BuyLots, SellLots;
   double         LowestBuy = 999, HighestBuy = 0.0001, LowestSell = 999, HighestSell = 0.0001, HighPoint, MidPoint, LowPoint, HighestBuyRec, LowestBuyRec, LowestSellRec, HighestSellRec;
   double         Profit = 0, BuyProfit = 0, SellProfit = 0, PosBuyProfit = 0, PosSellProfit = 0, ProfitRec = 0, BuyProfitRec = 0, SellProfitRec = 0, PosBuyProfitRec = 0, PosSellProfitRec = 0;
   int            HighestBuyTicket, LowestBuyTicket, HighestSellTicket, LowestSellTicket, HighestBuyTicketRec, LowestBuyTicketRec, HighestSellTicketRec, LowestSellTicketRec;
   double         LastRecSell, LastRecBuy, HighestBuyProfit, LowestBuyProfit, HighestSellProfit, LowestSellProfit, HighestBuyProfitRec, LowestBuyProfitRec, HighestSellProfitRec, LowestSellProfitRec;
   bool           SELLme = false;
   bool           BUYme = false;
   double         Margin = MarketInfo(Symbol(), MODE_MARGINREQUIRED);
   string         Message;

   

   for (Order = OrdersTotal() - 1; Order >= 0; Order--)
   {
      if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference)
         {
            
            Profit = OrderProfit() + OrderSwap() + OrderCommission();
            
            if (OrderType() == OP_BUY)
            {
               if (OrderOpenPrice() >= HighestBuy)
               {
                  HighestBuy = OrderOpenPrice();
                  HighestBuyTicket = OrderTicket();
                  HighestBuyProfit = Profit;
               }

               if (OrderOpenPrice() <= LowestBuy)
               {
                  LowestBuy = OrderOpenPrice();
                  LowestBuyTicket = OrderTicket();
                  LowestBuyProfit = Profit;
               }

               BuyOrders++;
               if (BuyOrders > MaxBuys) MaxBuys = BuyOrders;
               BuyLots += OrderLots();

               BuyProfit += Profit;
               if (Profit > 0) PosBuyProfit += Profit; 
               
            }

            if (OrderType() == OP_SELL)
            {
               if (OrderOpenPrice() <= LowestSell)
               {
                  LowestSell = OrderOpenPrice();
                  LowestSellTicket = OrderTicket();
                  LowestSellProfit = Profit;
               }

               if (OrderOpenPrice() >= HighestSell)
               {
                  HighestSell = OrderOpenPrice();
                  HighestSellTicket = OrderTicket();
                  HighestSellProfit = Profit;
               }

               SellOrders++;
               if (SellOrders > MaxSells) MaxSells = SellOrders;
               SellLots += OrderLots();

               SellProfit += Profit;
               if (Profit > 0) PosSellProfit += Profit; 
            }
         }        
         
          if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference *2)
         {
            
            ProfitRec = OrderProfit() + OrderSwap() + OrderCommission();
            
            if (OrderType() == OP_BUY)
            {
               if (OrderOpenPrice() >= HighestBuyRec)
               {
                  HighestBuyRec = OrderOpenPrice();
                  HighestBuyTicketRec = OrderTicket();
                  HighestBuyProfitRec = ProfitRec;
               }

               if (OrderOpenPrice() <= LowestBuyRec)
               {
                  LowestBuyRec = OrderOpenPrice();
                  LowestBuyTicketRec = OrderTicket();
                  LowestBuyProfitRec = ProfitRec;
               }
               if(OrderOpenTime() > LastRecBuy)
               {
                  LastRecBuy = OrderOpenTime();
               } 
               
               BuyOrdersRec++;
               if (BuyOrdersRec > MaxBuysRec) MaxBuysRec = BuyOrdersRec;

               BuyProfitRec += ProfitRec;
               if (ProfitRec > 0) PosBuyProfitRec += ProfitRec;
            }

            if (OrderType() == OP_SELL)
            {
               if (OrderOpenPrice() <= LowestSellRec)
               {
                  LowestSellRec = OrderOpenPrice();
                  LowestSellTicketRec = OrderTicket();
                  LowestSellProfitRec = ProfitRec;
               }

               if (OrderOpenPrice() >= HighestSellRec)
               {
                  HighestSellRec = OrderOpenPrice();
                  HighestSellTicketRec = OrderTicket();
                  HighestSellProfitRec = ProfitRec;
               }
               if(OrderOpenTime() > LastRecSell)
               {
                  LastRecSell = OrderOpenTime();
               } 

               SellOrdersRec++;
               if (SellOrdersRec > MaxSellsRec) MaxSellsRec = SellOrdersRec;

               SellProfitRec += ProfitRec;
               if (ProfitRec > 0) PosSellProfitRec += ProfitRec; 
            }
         }

      }
   }

      //BuyPipTarget = ProfitTarget;
      //SellPipTarget = ProfitTarget;

   if (HighestBuy >= HighestSell)
      HighPoint = HighestBuy;
   else
      HighPoint = HighestSell;

   if (LowestBuy <= LowestSell)
      LowPoint = LowestBuy;
   else
      LowPoint = LowestSell;


//   if ((SellOrders > 1 && BuyOrders > 0) || (SellOrders > 0 && BuyOrders > 1)) MidPoint = (HighPoint + LowPoint) / 2;

   RefreshRates();
//------------------------------------------------------------------------------------------------
 
//------------------------------------------------------------------------------------------------
// ---------------------------------- Take Profit code --------------------------------------

ProfitTarget=(iATR(NULL,240,750,1)+iATR(NULL,60,750,1))/Point/1.75/10;

double RecCount=CountedBuyRec()+CountedSellRec();
double savingacc;
double TotalProfitRec = BuyProfitRec + SellProfitRec; 
if(RecoveryOrders()==0)TotalProfitRec=0;

double CycleProfit;
if(BuyOrders+SellOrders!=0){
if(BuyOrders>0&&SellOrders==0)CycleProfit = TotalProfitRec+BuyCycle();
if(SellOrders>0&&BuyOrders==0)CycleProfit = TotalProfitRec+SellCycle();
if(SellOrders>0&&BuyOrders>0)CycleProfit=TotalProfitRec+SellCycle()+BuyCycle();} else CycleProfit=TotalProfitRec+SellCycle()+BuyCycle();

double BuyCycleProfit = BuyCycle()+SellProfitRec;
double SellCycleProfit = SellCycle()+BuyProfitRec;

  if(CycleProfit<-MaxLossAmount)savingacc=1; else savingacc=0;

 double TotalProfit = BuyProfit + SellProfit;
 double TotalOrders = BuyOrders + SellOrders;
 
 static double MaxDD;
  static double MaxBuyProfit;
  static double MaxSellProfit;

  if(CycleProfit<MaxDD)MaxDD=CycleProfit; 
  
  if(CycleProfit > MaxProfit) MaxProfit =CycleProfit;
  if(TotalOrders == 0)MaxProfit = 0;
 
   if(BuyCycleProfit > MaxBuyProfit) MaxBuyProfit =BuyCycleProfit;
  if(BuyOrders == 0)MaxBuyProfit = 0;

   if(SellCycleProfit > MaxSellProfit) MaxSellProfit =SellCycleProfit;
  if(SellOrders == 0)MaxSellProfit = 0;


 static double HighestMargin;
 if(AccountMargin()>HighestMargin)HighestMargin=AccountMargin();
 static double HighestBuyLots;
 static double HighestSellLots;
 if(BuyLots>HighestBuyLots)HighestBuyLots=BuyLots;
 if(SellLots>HighestSellLots)HighestSellLots=SellLots;


  static double SavedProfit;
  if(MaxProfit>ProfitTarget)SavedProfit=MaxProfit*((100-MaxRetrace)/100);else SavedProfit=-9999;
  if(TotalOrders == 0) SavedProfit=0;
  
    static double SavedBuyProfit;
  if(MaxBuyProfit>ProfitTarget)SavedBuyProfit=MaxBuyProfit*((100-MaxRetrace)/100);else SavedBuyProfit=-9999;
  if(BuyOrders == 0) SavedBuyProfit=0;

    static double SavedSellProfit;
  if(MaxSellProfit>ProfitTarget)SavedSellProfit=MaxSellProfit*((100-MaxRetrace)/100);else SavedSellProfit=-9999;
  if(SellOrders == 0) SavedSellProfit=0;
 
   
   double BreakEven;
   if(BuyOrders>0)BreakEven=(SumBuy()/BuyLots/10000000);
   if(SellOrders>0)BreakEven=(SumSell()/SellLots/10000000);
   if(TotalOrders==0)BreakEven=0;
   MidPoint = BreakEven;
   


double LastRec;

//------------------------------------
//------------------------------------

 if (ProfitMode==1 || ProfitMode==2 && BuyProfit + SellProfit >= ProfitTarget) 
 {
   for (Order = OrdersTotal() - 1; Order >= 0; Order--)
   {
    if (OrderSelect(Order, SELECT_BY_POS, MODE_TRADES))
    {
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference)            
      {
         OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Green);
      }   
    Error = GetLastError();
    if (Error != 0) Write("Error closing order " + OrderTicket() + ": " + ErrorDescription(Error) + " (F" + Error + ")  Lots:" + OrderLots() + "  Ask:" + MarketInfo(OrderSymbol(), MODE_ASK));    
    }
   }
 }
//------------------------------------
 if (ProfitMode==0 || ProfitMode==3)
 {
      if(BuyCycleProfit<=SavedBuyProfit){LastRec = TimeCurrent(); CloseAllBuyProfit();}
      if(SellCycleProfit<=SavedSellProfit){LastRec = TimeCurrent(); CloseAllSellProfit();}
 }

// ----------------------------------End Take Profit code --------------------------------------
    
    
   


   int Direction= Direction();
   if(ReverseDirection)  Direction = Direction;  
   
  double Directionh=Direction;  
        
      double hihi = iClose(NULL, 15, iHighest(NULL, 15, MODE_CLOSE, 16,0));
     double hihi2 = iClose(NULL, 15, iHighest(NULL, 15, MODE_CLOSE, 120,10));
     double lolo = iClose(NULL, 15, iLowest(NULL, 15, MODE_CLOSE, 16,0));
     double lolo2 = iClose(NULL, 15, iLowest(NULL, 15, MODE_CLOSE, 120,10));

// if(hihi>hihi2)Directionh=0; 
// if(lolo<lolo2)Directionh=0;

  string Market = "";
  
  if(hihi>hihi2||lolo<lolo2)Market="FREAKING MARKET WAITING"; else Market="ABLE TO TRADE";
  

// ---------------------------------- Loss Management code --------------------------------------
   bool NoBuy=false;
   bool NoSell=false;
double Recoverydirection;
double RecoveryOrdersvol;
if(TotalOrders==0)Recoverydirection=0;

for(int cnth = 0; cnth < OrdersTotal(); cnth++){
   OrderSelect(cnth, SELECT_BY_POS, MODE_TRADES);
     if(OrderMagicNumber() == Reference * 2)
       RecoveryOrdersvol = (OrderLots()+ RecoveryOrdersvol);}
      if(RecoveryOrders()==0)RecoveryOrdersvol=0;
 
 RecActivation=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/3;
double RecActivationh1;
if(CountedSellRec()==4)RecActivationh1=RecActivation*200;
if(CountedSellRec()==3)RecActivationh1=RecActivation*27;
if(CountedSellRec()==2)RecActivationh1=RecActivation*9;
if(CountedSellRec()==1)RecActivationh1=RecActivation*3;
if(CountedSellRec()==0)RecActivationh1=RecActivation*1;

double RecActivationh2;
if(CountedBuyRec()==4)RecActivationh2=RecActivation*200;
if(CountedBuyRec()==3)RecActivationh2=RecActivation*27;
if(CountedBuyRec()==2)RecActivationh2=RecActivation*9;
if(CountedBuyRec()==1)RecActivationh2=RecActivation*3;
if(CountedBuyRec()==0)RecActivationh2=RecActivation*1;


   if(AllowRecovery)
   {RefreshRates();
      int returnValue;
      
      double MaxLoss2 = AccountBalance()/100*MaxLossPercent2;


      if(SellCycleProfit<=-RecActivationh1 && SellLots*RecoveryLotMultiplier > RecoveryOrdersvol && Lastselltrade()>LastBuyRec()+100)
      { 
            NoSell=false; //Do not allow to place more losing orders
            NoBuy=false;
            Recoverydirection=1;
            
          if(ExitAllTrades) ExitAllTrades(Aqua,"Sell profit going under max allowed loss");
          
          if(PlaceRecoveryOrders && RecoveryOrders() < MaxRecoveryOrders)
          {
             //Calculate recovery lot size
          double recoverySLot=SellLots*RecoveryLotMultiplier;
          NormalizeDouble(recoverySLot,2);
          
          if(TimeCurrent()>LastBuyRec()+100) 
          OrderSend(Symbol(), OP_BUY,  recoverySLot, Ask, Slippage, Ask - RecoveryStopLoss * Point, 0, "Rec", Reference*2,0, DarkTurquoise);

          if(StopTrading) CeaseTrading=true;
          }
        }


         if(BuyCycleProfit<=-RecActivationh2 && BuyLots*RecoveryLotMultiplier > RecoveryOrdersvol && Lastbuytrade()>LastSellRec()+100)
         {
           NoBuy=false; //Do not allow to place more losing orders
           NoSell=false;
           Recoverydirection=2;

          if(ExitAllTrades)ExitAllTrades(Aqua,"Buy profit going under max allowed loss");
          
          if(PlaceRecoveryOrders && RecoveryOrders() < MaxRecoveryOrders)
          {
             //Calculate recovery lot size
          double recoveryBLot=BuyLots*RecoveryLotMultiplier;

          NormalizeDouble(recoveryBLot,2);
          if(TimeCurrent()>LastSellRec()+100)
          OrderSend(Symbol(), OP_SELL, recoveryBLot, Bid, Slippage, Bid + RecoveryStopLoss * Point,0, "Rec", Reference*2,0, DarkOrange);

          if(StopTrading) CeaseTrading=true;          
          }     
        }  
      }
// ----------------------------------End Loss Management code -------------------------------------   

// ----------------------------------Forced cycle start code -------------------------------------- 

   if(ForcedStart>0 && BuyOrders+SellOrders==0){
      if(ForcedStart==1 && Direction==-1 || ForcedStart==2 && Direction==1) TradeAllowed=false; else TradeAllowed=true;
}
// ----------------------------------End Forced cycle start code ----------------------------------

// ----------------------------------Variable spacing code ----------------------------------------

   if (AutoSpacing == 1){
      double SpacingModeATR;
      double b=(iATR(Symbol(),PERIOD_H1,750,0))/Point;
      double a=(iATR(Symbol(),PERIOD_M15,750,0))/Point;
      double c=(b/a);
      double small=(b/c);
      double e=(c*0.75);
      double great=(b/e);
      double acti = small;
      double acti2 = great; 
      double stddev3;
      double stddev4;
      double stddev5;
      double stddev6;
      double stddev;
      double spacingmin = (a/1.25);
      double stddev1 = iStdDev(Symbol(),StDevTF,StDevPer,0,StDevMode,PRICE_OPEN,0)/Point;
      double stddev2 = iStdDev(Symbol(),StDevTF2,StDevPer2,0,StDevMode2,PRICE_OPEN,0)/Point;
      if ((iATR(Symbol(),15,3,0))/Point > acti || (iATR(Symbol(),15,3,1))/Point > acti ||
          (iATR(Symbol(),15,3,2))/Point > acti || (iATR(Symbol(),15,3,3))/Point > acti ||
          (iATR(Symbol(),15,3,4))/Point > acti || (iATR(Symbol(),15,3,5))/Point > acti) stddev3 = (iATR(Symbol(),15,3,0))*1.5/Point; else stddev3=0;
      if ((iATR(Symbol(),15,3,0))/Point > acti || (iATR(Symbol(),15,3,1))/Point > acti ||
          (iATR(Symbol(),15,3,2))/Point > acti || (iATR(Symbol(),15,3,3))/Point > acti ||
          (iATR(Symbol(),15,3,4))/Point > acti || (iATR(Symbol(),15,3,5))/Point > acti) stddev4 = (iATR(Symbol(),15,3,1))*1.5/Point; else stddev4=0;
      if ((iATR(Symbol(),15,3,0))/Point > acti2 || (iATR(Symbol(),15,3,1))/Point > acti2 ||
          (iATR(Symbol(),15,3,2))/Point > acti2 || (iATR(Symbol(),15,3,3))/Point > acti2 ||
          (iATR(Symbol(),15,3,4))/Point > acti2 || (iATR(Symbol(),15,3,5))/Point > acti2) stddev5 = (iATR(Symbol(),15,3,0))*2.25/Point; else stddev5=0;
      if ((iATR(Symbol(),15,3,0))/Point > acti2 || (iATR(Symbol(),15,3,1))/Point > acti2 ||
          (iATR(Symbol(),15,3,2))/Point > acti2 || (iATR(Symbol(),15,3,3))/Point > acti2 ||
          (iATR(Symbol(),15,3,4))/Point > acti2 || (iATR(Symbol(),15,3,5))/Point > acti2) stddev6 = (iATR(Symbol(),15,3,1))*2.25/Point; else stddev6=0;
      double stddeva2 = MathMax(stddev5,stddev6);
      double stddeva = MathMax(stddev3,stddev4);
      double stddevs = MathMax(stddev1,stddev2);
      double stddevh = MathMax(stddeva,stddevs);
      double stddef = MathMax(stddevh,stddeva2);
      double sm = MathMax(stddeva,stddeva2);
      double reca=a*2;
      double recb=(iATR(Symbol(),15,3,1))*2.25/Point;
      if (stddef<spacingmin) stddev = spacingmin; else stddev = stddef;
      if(stddef<spacingmin) SpacingModeATR =9;
      else if (stddeva2<stddeva && stddeva > stddevs) SpacingModeATR = 1; 
      else if (stddeva2>stddeva && stddeva2> stddevs) SpacingModeATR = 2; 
      else if(stddevs>stddeva && stddevs>stddeva2) SpacingModeATR=0; 
      else SpacingModeATR = 999; 
      Spacing = stddev;
      NormalizeDouble(Spacing,0);
      //if(Spacing<15) Spacing=15;
      TrendSpacing=stddev;
   }
// ----------------------------------End Variable spacing code ------------------------------------- 


  
// ----------------------------------Open Trade code -----------------------------------------------      
double lotstaffel1=0.02;
double lotstaffel2=0.03;
double lotstaffel3=0.04;
double lotstaffel4=0.05;
double lotstaffel5=0.06;
double lotstaffel6=0.07;
double lotstaffel7=0.08;
double lotstaffel8=0.09;
double lotstaffel9=0.10;
double lotstaffel10=0.11;
double Sellloth1;
double Buyloth1;
double Buylothelp;
double Selllothelp;

if(BuyOrders==0)Buylothelp=lotstaffel1;
if(BuyOrders==1)Buylothelp=lotstaffel2;
if(BuyOrders==2)Buylothelp=lotstaffel3;
if(BuyOrders==3)Buylothelp=lotstaffel4;
if(BuyOrders==4)Buylothelp=lotstaffel5;
if(BuyOrders==5)Buylothelp=lotstaffel6;
if(BuyOrders==6)Buylothelp=lotstaffel7;
if(BuyOrders==7)Buylothelp=lotstaffel8;
if(BuyOrders==8)Buylothelp=lotstaffel9;
if(BuyOrders>=9)Buylothelp=lotstaffel10;

double Sellloth;
if(SellOrders==0)Selllothelp=lotstaffel1;
if(SellOrders==1)Selllothelp=lotstaffel2;
if(SellOrders==2)Selllothelp=lotstaffel3;
if(SellOrders==3)Selllothelp=lotstaffel4;
if(SellOrders==4)Selllothelp=lotstaffel5;
if(SellOrders==5)Selllothelp=lotstaffel6;
if(SellOrders==6)Selllothelp=lotstaffel7;
if(SellOrders==7)Selllothelp=lotstaffel8;
if(SellOrders==8)Selllothelp=lotstaffel9;
if(SellOrders>=9)Selllothelp=lotstaffel10;

if(increasebyloss==true && CycleProfit<-increaselimit)
 {
 Sellloth1=Selllothelp*increasefactor;
 Buyloth1=Buylothelp*increasefactor;
 }
if(increasebyloss==false || CycleProfit>-increaselimit)
 {
 Sellloth1=Selllothelp;
 Buyloth1=Buylothelp;
 }
 NormalizeDouble(Buyloth1,2);
 NormalizeDouble(Sellloth1,2);

if(Bid <= LowestSell)Sellloth1=lotstaffel1;
if(Ask >= HighestBuy)Buyloth1=lotstaffel1;


          double Selllotsoriginal;
          double Buylotsoriginal;

          double Selllotsoriginal1;
          double Buylotsoriginal1;
 double BuyLots11=NormalizeDouble(BuyLots,2);
  double SellLots11=NormalizeDouble(SellLots,2);


double AfterRecStartVolh;
if(-BuyCycleProfit<=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/3||CycleProfit>0)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/3)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/1.5)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2*1.5)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2*2)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-BuyCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2*2.5)RecoveryTakeProfit=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;

 
 double cycleh; 
 double CycleProfith;
 if (BuyCycleProfit<0)CycleProfith=-BuyCycleProfit; else if (BuyCycleProfit>=0)CycleProfith=BuyCycleProfit;
 
 if(BuyCycleProfit<-1000.00||BuyCycleProfit>1000.00)cycleh=(CycleProfith+ProfitTarget)/1000;
 if(BuyCycleProfit>-999.99&&BuyCycleProfit<999.99)cycleh=(CycleProfith+ProfitTarget)/100;
 if(BuyCycleProfit>-99.99&&BuyCycleProfit<99.99)cycleh=(CycleProfith+ProfitTarget)/10; 

 
 double cycleh1=NormalizeDouble(cycleh,1);
 double CycleProfit1;
 
 if(BuyCycleProfit<-1000.00||BuyCycleProfit>1000.00)CycleProfit1=cycleh1*100;
 if(BuyCycleProfit>-999.99&&BuyCycleProfit<999.99)CycleProfit1=cycleh1*10;
 if(BuyCycleProfit>-99.99&&BuyCycleProfit<99.99)CycleProfit1=cycleh1; 

AfterRecStartVolh=(CycleProfit1/RecoveryTakeProfit);


//------------------------------------------------
double AfterRecStartVolhs;
double RecoveryTakeProfits;
if(-SellCycleProfit<=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/3||CycleProfit>0)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/3)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/1.5)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2*1.5)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2*2)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;
if(-SellCycleProfit>(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point*2*2.5)RecoveryTakeProfits=(iATR(NULL,240,50,1)+iATR(NULL,60,50,1))/Point/2;

 
 double cyclehs; 
 double CycleProfiths;
 if (SellCycleProfit<0)CycleProfiths=-SellCycleProfit; else if (SellCycleProfit>=0)CycleProfiths=SellCycleProfit;
 
 if(SellCycleProfit<-1000.00||SellCycleProfit>1000.00)cyclehs=(CycleProfiths+ProfitTarget)/1000;
 if(SellCycleProfit>-999.99&&SellCycleProfit<999.99)cyclehs=(CycleProfiths+ProfitTarget)/100;
 if(SellCycleProfit>-99.99&&SellCycleProfit<99.99)cyclehs=(CycleProfiths+ProfitTarget)/10; 

 
 double cycleh1s=NormalizeDouble(cyclehs,1);
 double CycleProfit1s;
 
 if(SellCycleProfit<-1000.00||SellCycleProfit>1000.00)CycleProfit1s=cycleh1s*100;
 if(SellCycleProfit>-999.99&&SellCycleProfit<999.99)CycleProfit1s=cycleh1s*10;
 if(SellCycleProfit>-99.99&&SellCycleProfit<99.99)CycleProfit1s=cycleh1s; 

AfterRecStartVolhs=(CycleProfit1s/RecoveryTakeProfits);


//------------------------------------------------





Selllotsoriginal1=AfterRecStartVolhs-SellLots;
Buylotsoriginal1=AfterRecStartVolh-BuyLots;


if(Buylotsoriginal1<0.01)Buylotsoriginal=0.01;else Buylotsoriginal=Buylotsoriginal1;
if(Selllotsoriginal1<0.01)Selllotsoriginal=0.01; else Selllotsoriginal=Selllotsoriginal1;
if(Buylotsoriginal1>0.75)Buylotsoriginal=0.75;
if(Selllotsoriginal1>0.75)Selllotsoriginal=0.75;
NormalizeDouble(Selllotsoriginal,2);
NormalizeDouble(Buylotsoriginal,2);
   
          // ---------------------------------Order Lotsize  ---------------------------------

           
if(TradeAllowed){

double LowestBuy1=9999;
double HighestBuy1=0.00001;
double HighestSell1=0.00001;
double LowestSell1=9999;

double LowestBuyh;
double HighestBuyh;
double HighestSellh;
double LowestSellh;

if(LastBuyRec()>Lastselltrade()){
HighestSellh=HighestSell1;
LowestSellh=LowestSell1;} else{
HighestSellh=HighestSell;
LowestSellh=LowestSell;}

if(LastSellRec()>Lastbuytrade()){
HighestBuyh=HighestBuy1;
LowestBuyh=LowestBuy1;} else{
HighestBuyh=HighestBuy;
LowestBuyh=LowestBuy;}



// BUY Trade Criteria
   if (HighestBuy > 0 && LowestBuy < 1000)
   {
      if ((Ask <= LowestBuyh - (Spacing * Point)&&BuyOrders<MaximumBuyOrders) || Ask >= HighestBuyh + (TrendSpacing * Point)
           && BuyOrders<MaximumBuyOrders)
      {
         BUYme = true;
         if (OpenOnTick == 1 && TickPrice > 0 && Close[0] < TickPrice) BUYme = true;
      }
      if(AllowHedging==false&&SellOrders>0)BUYme = false;
      if(Directionh != 1)BUYme = false;
      if (CeaseTrading && BuyOrders == 0) BUYme = false;
      if(NoBuy==true)BUYme = false;
      if(LastSellRec()>TimeCurrent()-360)BUYme = false;

      if (PauseAtTime(StartTime,EndTime,1)!=1) BUYme = false;//Ask < Trend marketQuality()!=1
            if (SL == 0) stoploss = 0; else stoploss = Ask - (SL * Point);

      if (BUYme==true && TradeLong==true){  
      OrderSend(Symbol(), OP_BUY, Buylotsoriginal, Ask, Slippage, stoploss, 0, TradeComment, Reference,0, Blue);
      ExitSellRec(Yellow, "Next Cycle");
   Error = GetLastError();
   if (Error != 0)
      Write("Error opening BUY order: " + ErrorDescription(Error) + " (C" + Error + ")  Ask:" + Ask + "  Slippage:" + Slippage);}
   else
   {
      TickPrice = Close[0];
   }

   }

   // SELL Trade Criteria
   if (HighestSell > 0 && LowestSell < 1000)
   {
      if ((Bid >= HighestSellh + (Spacing * Point)&&SellOrders<MaximumSellOrders) || Bid <= LowestSellh - (TrendSpacing * Point)
          && SellOrders<MaximumSellOrders)
      {
         SELLme = true;
         if (OpenOnTick == 1 && TickPrice > 0 && Close[0] > TickPrice) SELLme = true;
      }
      if(AllowHedging==false&&BuyOrders>0)SELLme = false;
      if(Directionh != -1)SELLme = false;
      if (CeaseTrading && SellOrders == 0) SELLme = false;
      if(NoSell==true)SELLme = false;
      if (PauseAtTime(StartTime,EndTime,-1)!=-1) SELLme = false;//Bid > Trend marketQuality()!=-1      
      if(LastBuyRec()>TimeCurrent()-360)SELLme = false;
     
      if (SL == 0) stoploss = 0; else stoploss = Bid + (SL * Point);

      if (SELLme==true && TradeShort==true){
       OrderSend(Symbol(), OP_SELL, Selllotsoriginal, Bid, Slippage, stoploss, 0, TradeComment, Reference,0, Red);
      ExitBuyRec(Yellow, "Next Cycle");

   Error = GetLastError();
   if (Error != 0)
      Write("Error opening SELL order: " + ErrorDescription(Error) + " (D" + Error + ")  Bid:" + Bid + "  Slippage:" + Slippage);}
   else
   {
      TickPrice = Close[0];
   }

   }
 }  
// ----------------------------------End Open Trade code --------------------------------------   

   if(AccountMargin()!=0)
   {
      MarginPercent = MathRound((AccountEquity() / AccountMargin()) * 100);
   }   

   if (LowMarginPercent > MarginPercent) LowMarginPercent = MarginPercent;
   if (AccountEquity() < LowEquity) LowEquity = AccountEquity();

if(IsVisualMode() || !IsTesting())
{
   Message = "                  "+EA_name + NL +
             "                            ProfitTarget           " + DoubleToStr(ProfitTarget, 2) + NL +
             "                            Take Profit           " + DoubleToStr(RecoveryTakeProfit, 2) + NL +
             "                            Total P/L             " + DoubleToStr(CycleProfit, 2) + NL +
             "                            Buy Cycle Profit    " + DoubleToStr(BuyCycleProfit, 2) + "  Saved: " + DoubleToStr(SavedBuyProfit, 2) + NL + 
             "                            Sell Cycle Profit     " + DoubleToStr(SellCycleProfit, 2) + "  Saved: " + DoubleToStr(SavedSellProfit, 2) + NL + 
             "                            Almost Closed Rec    " + DoubleToStr(RecCount, 2) + NL +
             "                            SessionProfit          " + DoubleToStr(SessionProfit(), 2) + NL +
             "                            Buys                    " + BuyOrders + "  Highest: " + MaxBuys + NL +
             "                            BuyLots                " + DoubleToStr(BuyLots, 2) + "  Highest: " + DoubleToStr(HighestBuyLots, 2) + NL + 
             "                            BuyProfit              " + DoubleToStr(BuyProfit, 2) + NL +
             "                            Highest Buy           " + DoubleToStr(HighestBuyh, Digits) + " #" + DoubleToStr(HighestBuyTicket, 0) + "  Profit: " + DoubleToStr(HighestBuyProfit, 2) + NL +
             "                            Highest Sell           " + DoubleToStr(HighestSellh, Digits) + " #" + DoubleToStr(HighestSellTicket, 0) + "  Profit: " + DoubleToStr(HighestSellProfit, 2) + NL + NL +
             "                            Sells                     " + SellOrders + "  Highest: " + MaxSells + NL +
             "                            SellLots                 " + DoubleToStr(SellLots, 2) + "  Highest: " + DoubleToStr(HighestSellLots, 2) + NL + 
             "                            SellProfit               " + DoubleToStr(SellProfit, 2) + NL +
             "                            Lowest Buy            " + DoubleToStr(LowestBuyh, Digits) + " #" + DoubleToStr(LowestBuyTicket, 0) + "  Profit: " + DoubleToStr(LowestBuyProfit, 2) + NL +
             "                            Lowest Sell            " + DoubleToStr(LowestSellh, Digits) + " #" + DoubleToStr(LowestSellTicket, 0) + "  Profit: " + DoubleToStr(LowestSellProfit, 2) + NL + NL +
             "                            Spacing                 " + Spacing + NL +
             "                            SpacingModeATR    " + DoubleToStr(SpacingModeATR,0) + NL +NL+
             "                            Balance                " + DoubleToStr(AccountBalance(), 2) + NL +
             "                            Equity                  " + DoubleToStr(AccountEquity(), 2) + "  Lowest: " + DoubleToStr(LowEquity, 2) + NL + 
             "                            Max DD                " + DoubleToStr(MaxDD, 2) + NL + NL +             
             "                            Margin                  " + DoubleToStr(AccountMargin(), 2) + "  Highest: " + DoubleToStr(HighestMargin, 2) + NL + 
             "                            FreeMargin            " + DoubleToStr(AccountFreeMargin(), 2) + NL +
             "                            MarginPercent        " + DoubleToStr(MarginPercent, 2) + NL +
             "                            LowMarginPercent  " + DoubleToStr(LowMarginPercent, 2) + NL +
             "                            Current Time is      " +  TimeToStr(TimeCurrent(), TIME_SECONDS) + NL +
             "                            BreakEven           " +  DoubleToStr(BreakEven,4) + NL +
             "                            RecoveryLots         " +  DoubleToStr(RecoveryOrdersvol,2)  ;

   Comment(Message);
 
 //------------------------------------------------------------------------------------------------  
   if (RightSideLabel) 
   {
      string MarPercent = DoubleToStr(MarginPercent, 0);
      string LowMarPercent = DoubleToStr(LowMarginPercent, 0);

      string AcctBalance = DoubleToStr(AccountBalance(), 0);
      ObjectDelete("MarginPercent");

      if (ObjectFind("MarginPercent") != 0)
      {
         ObjectCreate("MarginPercent", OBJ_TEXT, 0, Time[0], Close[0]);
         ObjectSetText("MarginPercent", MarPercent + "%  " + LowMarPercent + "%  $" + AcctBalance, 10, "Arial Black", DodgerBlue);
      }
      else
      {
         ObjectMove("MarginPercent", 0, Time[0], Close[0]);
      }
   }

   if (ObjectFind("MidPoint") != 0)
   {
      ObjectCreate("MidPoint", OBJ_HLINE, 0, Time[0], MidPoint);
      ObjectSet("MidPoint", OBJPROP_COLOR, Gold);
      ObjectSet("MidPoint", OBJPROP_WIDTH, 1);
   }
   else
   {
      ObjectMove("MidPoint", 0, Time[0], MidPoint);
   }
}
 //------------------------------------------------------------------------------------------------    
   //QuitTrading(SellOrders);
   getSessionTarget();
   QuitAtTime(endDayHourMinute);
   return(0);
}
//------------------------------------------------------------------------------------------------

void Write(string String)
{
   int Handle;

   if (!Auditing) return;

   Handle = FileOpen(Filename, FILE_READ|FILE_WRITE|FILE_CSV, "/t");
   if (Handle < 1)
   {
      Print("Error opening audit file: Code ", GetLastError());
      return;
   }

   if (!FileSeek(Handle, 0, SEEK_END))
   {
      Print("Error seeking end of audit file: Code ", GetLastError());
      return;
   }

   if (FileWrite(Handle, TimeToStr(CurTime(), TIME_DATE|TIME_SECONDS) + "  " + String) < 1)
   {
      Print("Error writing to audit file: Code ", GetLastError());
      return;
   }

   FileClose(Handle);
}

//------------------------------------------------------------------------------------------------
int Direction() //tradeDirection=1: long, tradeDirection=-1: short 
{
int tradeDirection;

  if (((UseTMA && TMADecision() == 1) || !UseTMA) 
  && ((UseCCI && CCIDecision()  == 1) || !UseCCI) 
  && ((UseARSI && ARSIDecision()  == 1) || !UseARSI) 
  && ((Use_iTrend && iTrendDecision()  == 1) || !Use_iTrend))
   {
      tradeDirection=1;
   }
  if(((UseTMA && TMADecision() == -1)   || !UseTMA) 
  && ((UseCCI && CCIDecision()  == -1)  || !UseCCI) 
  && ((UseARSI && ARSIDecision() == -1) || !UseARSI) 
  && ((Use_iTrend && iTrendDecision()  == -1) || !Use_iTrend))
      tradeDirection=-1;
 
    if((TMADecision()==0 && CCIDecision()==0 && iTrendDecision()==0 && ARSIDecision()==0) 
    || (!UseTMA && !UseCCI && !Use_iTrend && !UseARSI))
      tradeDirection=0;
      
 
   return (tradeDirection);  
}

//------------------------------------------------------------------------------------------------
int TMADecision() //tradeDirection=1: long, tradeDirection=-1: short 
{
  int tradeDirection;
   
   if(UseTMA==True)
   {
      double MA3_0 = iMA(NULL, MAperiod, 2, 0, 3, PRICE_CLOSE, 0);
      double MA3_1 = iMA(NULL, MAperiod, 2, 0, 3, PRICE_CLOSE, 1);
      double MA20_0 = iMA(NULL, MAperiod, 3, 0, 3, PRICE_CLOSE, 0);
      double MA20_1 = iMA(NULL, MAperiod, 3, 0, 3, PRICE_CLOSE, 1);
      double MA50_0 = iMA(NULL, MAperiod, 4, 0, 3, PRICE_CLOSE, 0);
      double MA50_1 = iMA(NULL, MAperiod, 4, 0, 3, PRICE_CLOSE, 1);
      
     
      if (MA50_0<MA50_1 && MA20_0<MA20_1 && MA3_0<MA3_1 && MA3_0<MA20_0 && MA20_0<MA50_0) {tradeDirection=-1;} 
      if (MA50_0>MA50_1 && MA20_0>MA20_1 && MA3_0>MA3_1 && MA3_0>MA20_0 && MA20_0>MA50_0) {tradeDirection=1;} 
	}
	else
	tradeDirection=0;
	return(tradeDirection);
}
//------------------------------------------------------------------------------------------------
int CCIDecision()
{
 int tradeDirection;

   if(UseCCI==true)
   {
   double cci=0;
   cci=iCCI( NULL, CCI_Period, CCI_lenght, PRICE_TYPICAL, 0);
   if(cci < -cci_trigger)tradeDirection =-1;
    if(cci > cci_trigger)tradeDirection = 1;
   }
   else
   tradeDirection = 0;

   return(tradeDirection);         
}

int ARSIDecision()
{
 int tradeDirection;
double RSI_1, RSI_2, RSIT, RSHT, RSHTT, RSHTTT, HRSI_1, HRSI_2;
   if(UseARSI==true)
   {
      RSI_1 = iCustom(NULL, RSI_period,"Adaptive RSI", RSI_bars, PRICE_CLOSE, 0)*10;
      RSI_2 = iCustom(NULL, RSI_period,"Adaptive RSI", RSI_bars, PRICE_CLOSE, 1)*10;
      if(ARSI_trigger==0) RSIT = iATR(NULL,1,13, 0)*10;
      if(ARSI_trigger>0)  RSIT = ARSI_trigger;
      
       if ( RSI_1 > RSI_2 && RSI_1 - RSI_2 > RSIT) {tradeDirection=1;}
       if ( RSI_1 < RSI_2 && RSI_2 - RSI_1 > RSIT) {tradeDirection=-1;}   
   }
   else
   tradeDirection = 0;

   return(tradeDirection);         
}

int iTrendDecision()
{
int tradeDirection;
   if(Use_iTrend==true)
   {
      double B1_1 = iCustom(NULL, iTrendPeriod, "i_Trend", 0, 0, 0, 20, 2, 13, 300, 0, 0);
      double B1_2 = iCustom(NULL, iTrendPeriod, "i_Trend", 0, 0, 0, 20, 2, 13, 300, 1, 0);
      double B2_2 = iCustom(NULL, iTrendPeriod, "i_Trend", 0, 0, 0, 20, 2, 13, 300, 0, 1);
      
      double S1_1 = B1_1;
      double S1_2 = B1_2;
      double S2_2 = B2_2;
   
      if (B1_1 > B1_2 && B1_1 > B2_2) {tradeDirection=1;}
      if (S1_1 < S1_2 && S1_1 < S2_2) {tradeDirection=-1;}
 
   }     
   else
  tradeDirection=0;
   return(tradeDirection); 
}
//------------------------------------------------------------------------------------------------
void QuitAtTime(int endDayHour)//input example: close trading tomorrow at 20:59. Today: 14. April, toworrow:15.April. 142059
{
   int t=TimeDay(TimeCurrent())*10000+TimeHour(TimeCurrent())*100+TimeMinute(TimeCurrent());
   if(t>=endDayHour && endDayHour!=0)
   {
      CeaseTrading=true;
      ExitAllTrades(Gold, "Stopped Trading because endtime reached!");
   }
}

int PauseAtTime(int startTime, int endTime, int tradeDirection)
{
   int returnValue;
   if(startTime!=0 && endTime!=0)
   {
      int t=TimeDay(TimeCurrent())*10000+TimeHour(TimeCurrent())*100+TimeMinute(TimeCurrent());
      if(t>=startTime && t<=endTime)
      {
         returnValue=0;
      }
      else
      {
         if(tradeDirection==1){returnValue=1;}
         if(tradeDirection==-1){returnValue=-1;}
      }
   }
   else
   {
      if(tradeDirection==1){returnValue=1;}
      if(tradeDirection==-1){returnValue=-1;}      
   }
   return (returnValue);
}

double SessionProfit()
{
   double profitHistory;
   double profitOpenP;
   double returnValue;

   for(int ix = OrdersHistoryTotal()-1; ix>=0; ix--)
   {
      OrderSelect(ix,SELECT_BY_POS,MODE_HISTORY);
      if(OrderSymbol()==Symbol() && (OrderMagicNumber()==Reference || OrderMagicNumber()==Reference*2))
      {
            profitHistory +=OrderProfit(); 
      }
   }
   
   for(int j=OrdersTotal()-1;j>=0;j--)
   {
      OrderSelect(j,SELECT_BY_POS,MODE_TRADES);
      if(OrderSymbol()==Symbol() && (OrderMagicNumber()==Reference || OrderMagicNumber()==Reference*2))
      {
            profitOpenP +=OrderProfit(); 
      }      
   }
   returnValue=profitHistory+profitOpenP;   
   return(returnValue);         
}


double BuyCycle()
{
double OpenBuyProfits;
double ClosedRecProfitsS;
double OpenRecProfitsS;
double FirstBuyTime;
double ret;
// find earliest Buy
for(int im = OrdersTotal()-1; im >= 0; im--){
    OrderSelect(im, SELECT_BY_POS, MODE_TRADES);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference){
        if(OrderType() == OP_BUY){
            OpenBuyProfits += OrderProfit();
            if(OrderOpenTime() < FirstBuyTime || FirstBuyTime == 0){
                FirstBuyTime = OrderOpenTime();                
            }
        }    
     }
}
// find profits for closed SELL orders opened after the first BUY
for(int ima = OrdersHistoryTotal()-1; ima >= 0; ima--){
    OrderSelect(ima, SELECT_BY_POS, MODE_HISTORY);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference*2){
        if(OrderType() == OP_SELL){
            if(OrderOpenTime() >= FirstBuyTime && FirstBuyTime != 0){
                ClosedRecProfitsS += OrderProfit();
            }
        }  
    }
}

 ret=ClosedRecProfitsS+OpenBuyProfits+OpenRecProfitsS;
return(ret);
}

double SellCycle()
{
double OpenSellProfits;
double ClosedRecProfitsB;
double OpenRecProfitsB;
double FirstSellTime;
double reta;
// find earliest Sell
for(int imb = OrdersTotal()-1; imb >= 0; imb--){
    OrderSelect(imb, SELECT_BY_POS, MODE_TRADES);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference){

        if(OrderType() == OP_SELL){
            OpenSellProfits += OrderProfit();
            if(OrderOpenTime() < FirstSellTime || FirstSellTime==0){
                FirstSellTime = OrderOpenTime();                
            }
        }
   }
}
// find profits for Buy orders opened after the first Sell
for(int imc = OrdersHistoryTotal()-1; imc >= 0; imc--){
    OrderSelect(imc, SELECT_BY_POS, MODE_HISTORY);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference*2){
    
        if(OrderType() == OP_BUY){
            if(OrderOpenTime() >= FirstSellTime && FirstSellTime!=0){
                ClosedRecProfitsB += OrderProfit();
            }
        }

    }
}
reta=ClosedRecProfitsB+OpenSellProfits+OpenRecProfitsB;
return(reta);
}

double CountedBuyRec()
{
int CountedRec=0;
double FirstBuyTimerec;
// find earliest Buy
for(int imy = OrdersTotal()-1; imy >= 0; imy--){
    OrderSelect(imy, SELECT_BY_POS, MODE_TRADES);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference){
        if(OrderType() == OP_BUY){
            if(OrderOpenTime() < FirstBuyTimerec || FirstBuyTimerec == 0){
                FirstBuyTimerec = OrderOpenTime();                
            }
        }    
     }
}
// find profits for closed SELL orders opened after the first BUY
for(int imay = OrdersHistoryTotal()-1; imay >= 0; imay--){
    OrderSelect(imay, SELECT_BY_POS, MODE_HISTORY);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference*2){
        if(OrderType() == OP_SELL){
            if(OrderOpenTime() > FirstBuyTimerec && FirstBuyTimerec != 0){
                CountedRec++; 
            }
        }
    }
}
return(CountedRec);
}

double CountedSellRec()
{
int CountedReca=0;
double FirstSellTimerec;
// find earliest Sell
for(int imby = OrdersTotal()-1; imby >= 0; imby--){
    OrderSelect(imby, SELECT_BY_POS, MODE_TRADES);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference){

        if(OrderType() == OP_SELL){
            if(OrderOpenTime() < FirstSellTimerec || FirstSellTimerec==0){
                FirstSellTimerec = OrderOpenTime();                
            }
        }
   }
}
// find profits for Buy orders opened after the first Sell
for(int imcy = OrdersHistoryTotal()-1; imcy >= 0; imcy--){
    OrderSelect(imcy, SELECT_BY_POS, MODE_HISTORY);
    if(OrderSymbol() == Symbol() && OrderMagicNumber() == Reference*2){
    
        if(OrderType() == OP_BUY){
            if(OrderOpenTime() > FirstSellTimerec && FirstSellTimerec!=0){
                 CountedReca++;          
            }
        }

    }
}
return(CountedReca);
}
//------------------------------------------------------------------------------------------------
void getSessionTarget()
{
   if(SessionTarget>0){
      if(SessionProfit()>=SessionTarget)
      {
         CeaseTrading=true;
         ExitAllTrades(Aqua, "Session Target Achieved. YUHUUI!");           
      }
   }   
}

void ExitAllTrades(color Color, string reason){
   bool success;
   for (int cnt = OrdersTotal() - 1; cnt >= 0; cnt --){
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference){
      if (TimeCurrent()-OrderOpenTime() >= CloseDelay){
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Color);}
         if(success==true){
         if(!IsTesting())   Print("Closed all positions because ",reason);
         }
      }  
     }} 
void ExitAllRecTrades(color Color, string reason){
   bool success;
   for (int cntr = OrdersTotal() - 1; cntr >= 0; cntr --){
      OrderSelect(cntr, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Reference*2){
      if (TimeCurrent()-OrderOpenTime() >= CloseDelay){
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Color);}
         if(success==true){
         if(!IsTesting())   Print("Closed all positions because ",reason);
         }
      }   

   }
}   
void ExitBuyCycle(color Color, string reason){
   bool success;
   for (int cntr = OrdersTotal() - 1; cntr >= 0; cntr --){
      OrderSelect(cntr, SELECT_BY_POS, MODE_TRADES);
      if (OrderType()==OP_BUY && OrderSymbol() == Symbol() && (OrderMagicNumber() == Reference*2||OrderMagicNumber() == Reference)){
      if (TimeCurrent()-OrderOpenTime() >= CloseDelay){
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Color);}
         if(success==true){
         if(!IsTesting())   Print("Closed all positions because ",reason);
         }
      }   

   }
}   
void ExitSellCycle(color Color, string reason){
   bool success;
   for (int cntr = OrdersTotal() - 1; cntr >= 0; cntr --){
      OrderSelect(cntr, SELECT_BY_POS, MODE_TRADES);
      if (OrderType()==OP_SELL && OrderSymbol() == Symbol() && (OrderMagicNumber() == Reference*2||OrderMagicNumber() == Reference)){
      if (TimeCurrent()-OrderOpenTime() >= CloseDelay){
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Color);}
         if(success==true){
         if(!IsTesting())   Print("Closed all positions because ",reason);
         }
      }   

   }
} 
void ExitBuyRec(color Color, string reason){
   bool success;
   for (int cntr = OrdersTotal() - 1; cntr >= 0; cntr --){
      OrderSelect(cntr, SELECT_BY_POS, MODE_TRADES);
      if (OrderType()==OP_BUY && OrderSymbol() == Symbol() && (OrderMagicNumber() == Reference*2)){
      if (TimeCurrent()-OrderOpenTime() >= CloseDelay){
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Color);}
         if(success==true){
         if(!IsTesting())   Print("Closed all positions because ",reason);
         }
      }   

   }
}   
void ExitSellRec(color Color, string reason){
   bool success;
   for (int cntr = OrdersTotal() - 1; cntr >= 0; cntr --){
      OrderSelect(cntr, SELECT_BY_POS, MODE_TRADES);
      if (OrderType()==OP_SELL && OrderSymbol() == Symbol() && (OrderMagicNumber() == Reference*2)){
      if (TimeCurrent()-OrderOpenTime() >= CloseDelay){
         success=OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage, Color);}
         if(success==true){
         if(!IsTesting())   Print("Closed all positions because ",reason);
         }
      }   

   }
}   
  


//-----------------------------------------END-----------------------------------------------------



